<?php

namespace Drupal\entrasync\Plugin\QueueWorker;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Password\PasswordGeneratorInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\entrasync\Entity\SyncEntity;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes Entra users.
 *
 * @QueueWorker(
 *   id = "entra_user_processor",
 *   title = @Translation("Entra User Processor"),
 *   cron = {"time" = 60}
 * )
 */
class EntraUserProcessor extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The config.
   *
   * @var Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;


  /**
   * The password generator.
   *
   * @var \Drupal\Core\Password\PasswordGeneratorInterface
   */
  protected $passwordGenerator;

  /**
   * {@inheritdoc}
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory.
   * @param \Drupal\Core\Password\PasswordGeneratorInterface $passwordGenerator
   *
   */
  public function __construct($configuration,
                              $plugin_id,
                              $plugin_definition,
                              LoggerChannelFactoryInterface $loggerFactory,
                              PasswordGeneratorInterface $passwordGenerator) {
    // $this->configFactory = $configFactory;
    // $this->config = $configFactory->get('entrasync.settings'); // this is wrong, needs to use the plugin id
    $this->logger = $loggerFactory->get('entrasync');
    $this->passwordGenerator = $passwordGenerator;
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('password_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data)
  {
    if (!isset($data['user'], $data['sync_entity_id'])) {
      $this->logger->error('Queue item is missing required data.');
      return;
    }

    // Loading config and user data.
    $syncEntityId = $data['sync_entity_id'];

    /** @var \Drupal\entrasync\Entity\SyncEntity $syncEntity */
    $syncEntity = SyncEntity::load($syncEntityId);
    if (!$syncEntity) {
      $this->logger->error('Sync entity could not be loaded for ID: ' . $syncEntityId);
      return;
    }

    // Check if there is an email field on the Entra user
    if (empty($data['user']['email'])) {
      $this->logger->error('Empty email field for user: ' . $data['user']['userprincipalname'] . ' in ' . $syncEntityId . '. User not added to Drupal');
      return;
    }

    // Check if the user exist in Drupal
    $user_exists = user_load_by_mail($data['user']['email']);

    try {
      if ($user_exists === false) {
        // User doesn't exist, create it.
        $user = User::create();
        $user->setPassword($this->passwordGenerator->generate());
        $user->enforceIsNew();
        $user->setEmail($data['user']['email']);
        $user->setUsername($data['user']['userprincipalname']);
      } else {
        // User exists, load and update it.
        $user = $user_exists;
      }

      // Apply shared user data logic.
      $this->applyUserData($user, $data, $syncEntity);
    } catch (\Exception $e) {
      $this->logger->error('User processing failed: ' . $e->getMessage());
    }
  }

  protected function applyUserData(&$user, $data, $syncEntity) {
    // Set custom fields based on the mapping.
    $user_field_mapping = $syncEntity->get('user_field_mapping');
    foreach ($user_field_mapping as $entra_field => $drupal_field) {
      if (isset($data['user'][$entra_field]) && $data['user'][$entra_field] !== '') {
        $field_value = is_array($data['user'][$entra_field]) ? implode(', ', $data['user'][$entra_field]) : $data['user'][$entra_field];
        if ($user->hasField($drupal_field)) {
          $user->set($drupal_field, $field_value);
        } else {
          $this->logger->error('The field ' . $drupal_field . ' does not exist on the user entity.');
        }
      }
    }

    // Handle roles.
    $roles_to_modify = $syncEntity->get('modify_entrauser_roles');
    foreach ($roles_to_modify as $role_id) {
      if (!$user->hasRole($role_id)) {
        $user->addRole($role_id);
      }
      // @todo Additional logic for removing roles should be added here
    }

    // Handle activation status based on the syncEntity configuration.
    if ($syncEntity->get('entrauser_status') === 'active') {
      $user->activate();
    } else {
      $user->block();
    }

    // Save changes.
    $user->save();

    // Send email notification if applicable.
    if ($user->isNew() && $syncEntity->get('send_mail_on_activate')) {
      _user_mail_notify('register_admin_created', $user);
    }
  }

}
